/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.lucene.tests.search;

import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import org.apache.lucene.search.Matches;
import org.apache.lucene.search.MatchesIterator;

/** An implementation of {@link Matches} with additional consistency checks. */
public class AssertingMatches implements Matches {

  private final Matches in;

  public AssertingMatches(Matches matches) {
    this.in = matches;
  }

  @Override
  public MatchesIterator getMatches(String field) throws IOException {
    MatchesIterator mi = in.getMatches(field);
    if (mi == null) return null;
    return new AssertingMatchesIterator(mi);
  }

  @Override
  public Collection<Matches> getSubMatches() {
    return Collections.singleton(in);
  }

  @Override
  public Iterator<String> iterator() {
    return in.iterator();
  }

  public static Matches unWrap(Matches m) {
    while (m instanceof AssertingMatches) {
      m = (((AssertingMatches) m).in);
    }
    return m;
  }
}
